@@ -42,7 +42,6 @@ WINE_DECLARE_DEBUG_CHANNEL(gecko);
 
 #define NS_APPSTARTUPNOTIFIER_CONTRACTID "@mozilla.org/embedcomp/appstartup-notifier;1"
 #define NS_WEBBROWSER_CONTRACTID "@mozilla.org/embedding/browser/nsWebBrowser;1"
-#define NS_MEMORY_CONTRACTID "@mozilla.org/xpcom/memory-service;1"
 #define NS_COMMANDPARAMS_CONTRACTID "@mozilla.org/embedcomp/command-params;1"
 #define NS_HTMLSERIALIZER_CONTRACTID "@mozilla.org/layout/contentserializer;1?mimetype=text/html"
 #define NS_EDITORCONTROLLER_CONTRACTID "@mozilla.org/editor/editorcontroller;1"
@@ -69,13 +68,14 @@ static nsresult (CDECL *NS_CStringSetData)(nsACString*,const char*,PRUint32);
 static nsresult (CDECL *NS_NewLocalFile)(const nsAString*,cpp_bool,nsIFile**);
 static PRUint32 (CDECL *NS_StringGetData)(const nsAString*,const PRUnichar **,cpp_bool*);
 static PRUint32 (CDECL *NS_CStringGetData)(const nsACString*,const char**,cpp_bool*);
+static void* (CDECL *NS_Alloc)(SIZE_T);
+static void (CDECL *NS_Free)(void*);
 
 static HINSTANCE xul_handle = NULL;
 
 static nsIServiceManager *pServMgr = NULL;
 static nsIComponentManager *pCompMgr = NULL;
 static nsICategoryManager *cat_mgr;
-static nsIMemory *nsmem = NULL;
 static nsIFile *profile_directory, *plugin_directory;
 
 static const WCHAR wszNsContainer[] = {'N','s','C','o','n','t','a','i','n','e','r',0};
@@ -516,18 +516,11 @@ static BOOL load_xul(const PRUnichar *gre_path)
     NS_DLSYM(NS_NewLocalFile);
     NS_DLSYM(NS_StringGetData);
     NS_DLSYM(NS_CStringGetData);
-
-#undef NS_DLSYM
-
-#define NS_DLSYM(func) \
-    func = (void *)GetProcAddress(xul_handle, #func); \
-    if(!func) \
-        ERR("Could not GetProcAddress(" #func ") failed\n")
-
+    NS_DLSYM(NS_Alloc);
+    NS_DLSYM(NS_Free);
     NS_DLSYM(ccref_incr);
     NS_DLSYM(ccref_decr);
     NS_DLSYM(ccref_init);
-    NS_DLSYM(ccref_unmark_if_purple);
     NS_DLSYM(ccp_init);
     NS_DLSYM(describe_cc_node);
     NS_DLSYM(note_cc_edge);
@@ -734,11 +727,6 @@ static BOOL init_xpcom(const PRUnichar *gre_path)
     init_mutation(pCompMgr);
     set_preferences();
 
-    nsres = nsIComponentManager_CreateInstanceByContractID(pCompMgr, NS_MEMORY_CONTRACTID,
-            NULL, &IID_nsIMemory, (void**)&nsmem);
-    if(NS_FAILED(nsres))
-        ERR("Could not get nsIMemory: %08x\n", nsres);
-
     nsres = nsIServiceManager_GetServiceByContractID(pServMgr, NS_CATEGORYMANAGER_CONTRACTID,
             &IID_nsICategoryManager, (void**)&cat_mgr);
     if(NS_FAILED(nsres))
@@ -797,12 +785,12 @@ BOOL load_gecko(void)
 
 void *nsalloc(size_t size)
 {
-    return nsIMemory_Alloc(nsmem, size);
+    return NS_Alloc(size);
 }
 
 void nsfree(void *mem)
 {
-    nsIMemory_Free(nsmem, mem);
+    NS_Free(mem);
 }
 
 static BOOL nsACString_Init(nsACString *str, const char *data)
@@ -1119,9 +1107,6 @@ void close_gecko(void)
     if(cat_mgr)
         nsICategoryManager_Release(cat_mgr);
 
-    if(nsmem)
-        nsIMemory_Release(nsmem);
-
     /* Gecko doesn't really support being unloaded */
     /* if (hXPCOM) FreeLibrary(hXPCOM); */
 
@@ -1150,6 +1135,28 @@ BOOL is_gecko_path(const char *path)
     return ret;
 }
 
+void set_viewer_zoom(NSContainer *nscontainer, float factor)
+{
+    nsIContentViewer *content_viewer;
+    nsIDocShell *doc_shell;
+    nsresult nsres;
+
+    TRACE("Setting to %f\n", factor);
+
+    nsres = get_nsinterface((nsISupports*)nscontainer->navigation, &IID_nsIDocShell, (void**)&doc_shell);
+    assert(nsres == NS_OK);
+
+    nsres = nsIDocShell_GetContentViewer(doc_shell, &content_viewer);
+    assert(nsres == NS_OK && content_viewer);
+    nsIDocShell_Release(doc_shell);
+
+    nsres = nsIContentViewer_SetFullZoom(content_viewer, factor);
+    if(NS_FAILED(nsres))
+        ERR("SetFullZoom failed: %08x\n", nsres);
+
+    nsIContentViewer_Release(content_viewer);
+}
+
 struct nsWeakReference {
     nsIWeakReference nsIWeakReference_iface;
 
@@ -1469,9 +1476,21 @@ static nsresult NSAPI nsContextMenuListener_OnShowContextMenu(nsIContextMenuList
     switch(aContextFlags) {
     case CONTEXT_NONE:
     case CONTEXT_DOCUMENT:
-    case CONTEXT_TEXT:
-        dwID = CONTEXT_MENU_DEFAULT;
+    case CONTEXT_TEXT: {
+        nsISelection *selection;
+
+        nsres = nsIDOMHTMLDocument_GetSelection(This->doc->basedoc.doc_node->nsdoc, &selection);
+        if(NS_SUCCEEDED(nsres) && selection) {
+            cpp_bool is_collapsed;
+
+            /* FIXME: Check if the click was inside selection. */
+            nsres = nsISelection_GetIsCollapsed(selection, &is_collapsed);
+            nsISelection_Release(selection);
+            if(NS_SUCCEEDED(nsres) && !is_collapsed)
+                dwID = CONTEXT_MENU_TEXTSELECT;
+        }
         break;
+    }
     case CONTEXT_IMAGE:
     case CONTEXT_IMAGE|CONTEXT_LINK:
         dwID = CONTEXT_MENU_IMAGE;
@@ -1713,8 +1732,24 @@ static nsresult NSAPI nsEmbeddingSiteWindow_GetDimensions(nsIEmbeddingSiteWindow
         UINT32 flags, LONG *x, LONG *y, LONG *cx, LONG *cy)
 {
     NSContainer *This = impl_from_nsIEmbeddingSiteWindow(iface);
-    WARN("(%p)->(%08x %p %p %p %p)\n", This, flags, x, y, cx, cy);
-    return NS_ERROR_NOT_IMPLEMENTED;
+    RECT r;
+
+    TRACE("(%p)->(%x %p %p %p %p)\n", This, flags, x, y, cx, cy);
+
+    if(!GetWindowRect(This->hwnd, &r)) {
+        ERR("GetWindowRect failed\n");
+        return NS_ERROR_FAILURE;
+    }
+
+    if(x)
+        *x = r.left;
+    if(y)
+        *y = r.top;
+    if(cx)
+        *cx = r.right-r.left;
+    if(cy)
+        *cy = r.bottom-r.top;
+    return NS_OK;
 }
 
 static nsresult NSAPI nsEmbeddingSiteWindow_SetFocus(nsIEmbeddingSiteWindow *iface)
