@@ -98,7 +98,6 @@ typedef struct {
 } xy_test_t;
 
 static const xy_test_t no_xy = {-10,-10,-10,-10};
-static const xy_test_t zero_xy = {0,0,0,0};
 
 static const char empty_doc_str[] =
     "<html></html>";
@@ -1058,6 +1057,7 @@ static HRESULT WINAPI submit_onclick_setret(IDispatchEx *iface, DISPID id, LCID
         VARIANT *pvarRes, EXCEPINFO *pei, IServiceProvider *pspCaller)
 {
     IHTMLEventObj *event;
+    VARIANT v;
     HRESULT hres;
 
     CHECK_EXPECT(submit_onclick_setret);
@@ -1069,10 +1069,20 @@ static HRESULT WINAPI submit_onclick_setret(IDispatchEx *iface, DISPID id, LCID
     ok(hres == S_OK, "get_event failed: %08x\n", hres);
     ok(event != NULL, "event == NULL\n");
 
+    V_VT(&v) = VT_ERROR;
+    hres = IHTMLEventObj_get_returnValue(event, &v);
+    ok(hres == S_OK, "get_returnValue failed: %08x\n", hres);
+    ok(V_VT(&v) == VT_EMPTY, "V_VT(returnValue) = %d\n", V_VT(&v));
+
     hres = IHTMLEventObj_put_returnValue(event, onclick_event_retval);
     ok(hres == S_OK, "put_returnValue failed: %08x\n", hres);
     IHTMLEventObj_Release(event);
 
+    V_VT(&v) = VT_ERROR;
+    hres = IHTMLEventObj_get_returnValue(event, &v);
+    ok(hres == S_OK, "get_returnValue failed: %08x\n", hres);
+    ok(VarCmp(&v, &onclick_event_retval, 0, 0) == VARCMP_EQ, "unexpected returnValue\n");
+
     *pvarRes = onclick_retval;
     return S_OK;
 }
@@ -1413,7 +1423,7 @@ static void pump_msgs(BOOL *b)
             DispatchMessageW(&msg);
         }
     }else {
-        while(!b && PeekMessageW(&msg, NULL, 0, 0, PM_REMOVE)) {
+        while(PeekMessageW(&msg, NULL, 0, 0, PM_REMOVE)) {
             TranslateMessage(&msg);
             DispatchMessageW(&msg);
         }
@@ -1572,7 +1582,7 @@ static void test_onclick(IHTMLDocument2 *doc)
     V_VT(&v) = VT_BSTR;
     V_BSTR(&v) = a2bstr("function();");
     hres = IHTMLElement_put_onclick(div, v);
-    todo_wine ok(hres == S_OK, "put_onclick failed: %08x\n", hres);
+    ok(hres == S_OK, "put_onclick failed: %08x\n", hres);
 
     if(hres == S_OK) {
         V_VT(&v) = VT_EMPTY;
