@@ -77,6 +77,17 @@ static inline IQueryAssociationsImpl *impl_from_IQueryAssociations(IQueryAssocia
   return CONTAINING_RECORD(iface, IQueryAssociationsImpl, IQueryAssociations_iface);
 }
 
+struct enumassochandlers
+{
+    IEnumAssocHandlers IEnumAssocHandlers_iface;
+    LONG ref;
+};
+
+static inline struct enumassochandlers *impl_from_IEnumAssocHandlers(IEnumAssocHandlers *iface)
+{
+  return CONTAINING_RECORD(iface, struct enumassochandlers, IEnumAssocHandlers_iface);
+}
+
 /**************************************************************************
  *  IQueryAssociations_QueryInterface
  *
@@ -862,11 +873,105 @@ static ULONG WINAPI ApplicationAssociationRegistration_Release(IApplicationAssoc
     return ref;
 }
 
-static HRESULT WINAPI ApplicationAssociationRegistration_QueryCurrentDefault(IApplicationAssociationRegistration* This, LPCWSTR query,
+static HRESULT WINAPI ApplicationAssociationRegistration_QueryCurrentDefault(IApplicationAssociationRegistration *iface, LPCWSTR query,
                                                                              ASSOCIATIONTYPE type, ASSOCIATIONLEVEL level, LPWSTR *association)
 {
-    FIXME("(%p)->(%s, %d, %d, %p)\n", This, debugstr_w(query), type, level, association);
-    return E_NOTIMPL;
+    IApplicationAssociationRegistrationImpl *This = impl_from_IApplicationAssociationRegistration(iface);
+    static WCHAR urlassoc[] = {'U','r','l','A','s','s','o','c','i','a','t','i','o','n','s',0};
+    static WCHAR mimeassoc[] = {'M','I','M','E','A','s','s','o','c','i','a','t','i','o','n','s',0};
+    static WCHAR assocations[] = {'S','o','f','t','w','a','r','e','\\','M','i','c','r','o','s','o','f','t','\\',
+                                'W','i','n','d','o','w','s','\\','S','h','e','l','l','\\',
+                                'A','s','s','o','c','i','a','t','i','o','n','s',0};
+    static WCHAR slash[] = {'\\',0};
+    static WCHAR choice[] = {'U','s','e','r','C','h','o','i','c','e',0};
+    static WCHAR propid[] = {'P','r','o','g','i','d',0};
+    WCHAR path[MAX_PATH] = {0};
+    DWORD ret, keytype, size;
+    HKEY hkey = NULL;
+    HRESULT hr = HRESULT_FROM_WIN32(ERROR_NO_ASSOCIATION);
+
+    TRACE("(%p)->(%s, %d, %d, %p)\n", This, debugstr_w(query), type, level, association);
+
+    if(!association)
+        return E_INVALIDARG;
+
+    *association = NULL;
+
+    if((type == AT_URLPROTOCOL || type == AT_FILEEXTENSION) && !query[0])
+        return E_INVALIDARG;
+    else if(type == AT_FILEEXTENSION && query[0] != '.')
+        return E_INVALIDARG;
+
+    if(type == AT_FILEEXTENSION)
+    {
+        ret = RegOpenKeyExW(HKEY_CLASSES_ROOT, query, 0, KEY_READ, &hkey);
+        if(ret == ERROR_SUCCESS)
+        {
+            ret = RegGetValueW(hkey, NULL, NULL, RRF_RT_REG_SZ, &keytype, NULL, &size);
+            if(ret == ERROR_SUCCESS)
+            {
+                *association = CoTaskMemAlloc(size);
+                if(*association)
+                {
+                    ret = RegGetValueW(hkey, NULL, NULL, RRF_RT_REG_SZ, &keytype, *association, &size);
+                    if(ret == ERROR_SUCCESS)
+                        hr = S_OK;
+                    else
+                    {
+                        CoTaskMemFree(*association);
+                        *association = NULL;
+                    }
+                }
+                else
+                    hr = E_OUTOFMEMORY;
+            }
+        }
+    }
+    else
+    {
+        ret = RegOpenKeyExW(HKEY_CURRENT_USER, assocations, 0, KEY_READ, &hkey);
+        if(ret == ERROR_SUCCESS)
+        {
+            if(type == AT_URLPROTOCOL)
+                lstrcpyW(path, urlassoc);
+            else if(type == AT_MIMETYPE)
+                lstrcpyW(path, mimeassoc);
+            else
+            {
+                WARN("Unsupported type (%d).\n", type);
+                RegCloseKey(hkey);
+                return hr;
+            }
+
+            lstrcatW(path, slash);
+            lstrcatW(path, query);
+            lstrcatW(path, slash);
+            lstrcatW(path, choice);
+
+            ret = RegGetValueW(hkey, path, propid, RRF_RT_REG_SZ, &keytype, NULL, &size);
+            if(ret == ERROR_SUCCESS)
+            {
+                *association = CoTaskMemAlloc(size);
+                if(*association)
+                {
+                    ret = RegGetValueW(hkey, path, propid, RRF_RT_REG_SZ, &keytype, *association, &size);
+                    if(ret == ERROR_SUCCESS)
+                        hr = S_OK;
+                    else
+                    {
+                        CoTaskMemFree(*association);
+                        *association = NULL;
+                    }
+                }
+                else
+                    hr = E_OUTOFMEMORY;
+            }
+        }
+    }
+
+    RegCloseKey(hkey);
+
+    return hr;
 }
 
 static HRESULT WINAPI ApplicationAssociationRegistration_QueryAppIsDefault(IApplicationAssociationRegistration* This, LPCWSTR query,
@@ -965,3 +1070,82 @@ HRESULT WINAPI ApplicationAssociationRegistration_Constructor(IUnknown *outer, R
     TRACE("returning 0x%x with %p\n", hr, *ppv);
     return hr;
 }
+
+static HRESULT WINAPI enumassochandlers_QueryInterface(IEnumAssocHandlers *iface, REFIID riid, void **obj)
+{
+    struct enumassochandlers *This = impl_from_IEnumAssocHandlers(iface);
+
+    TRACE("(%p %s %p)\n", This, debugstr_guid(riid), obj);
+
+    if (IsEqualIID(riid, &IID_IEnumAssocHandlers) ||
+        IsEqualIID(riid, &IID_IUnknown))
+    {
+        *obj = iface;
+        IEnumAssocHandlers_AddRef(iface);
+        return S_OK;
+    }
+
+    *obj = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI enumassochandlers_AddRef(IEnumAssocHandlers *iface)
+{
+    struct enumassochandlers *This = impl_from_IEnumAssocHandlers(iface);
+    ULONG ref = InterlockedIncrement(&This->ref);
+
+    TRACE("(%p)->(%u)\n", This, ref);
+    return ref;
+}
+
+static ULONG WINAPI enumassochandlers_Release(IEnumAssocHandlers *iface)
+{
+    struct enumassochandlers *This = impl_from_IEnumAssocHandlers(iface);
+    ULONG ref = InterlockedDecrement(&This->ref);
+
+    TRACE("(%p)->(%u)\n", This, ref);
+
+    if (!ref)
+        SHFree(This);
+
+    return ref;
+}
+
+static HRESULT WINAPI enumassochandlers_Next(IEnumAssocHandlers *iface, ULONG count, IAssocHandler **handlers,
+    ULONG *fetched)
+{
+    struct enumassochandlers *This = impl_from_IEnumAssocHandlers(iface);
+
+    FIXME("(%p)->(%u %p %p): stub\n", This, count, handlers, fetched);
+
+    return E_NOTIMPL;
+}
+
+static const IEnumAssocHandlersVtbl enumassochandlersvtbl = {
+    enumassochandlers_QueryInterface,
+    enumassochandlers_AddRef,
+    enumassochandlers_Release,
+    enumassochandlers_Next
+};
+
+/**************************************************************************
+ * SHAssocEnumHandlers            [SHELL32.@]
+ */
+HRESULT WINAPI SHAssocEnumHandlers(const WCHAR *extra, ASSOC_FILTER filter, IEnumAssocHandlers **enumhandlers)
+{
+    struct enumassochandlers *enumassoc;
+
+    FIXME("(%s %d %p): stub\n", debugstr_w(extra), filter, enumhandlers);
+
+    *enumhandlers = NULL;
+
+    enumassoc = SHAlloc(sizeof(*enumassoc));
+    if (!enumassoc)
+        return E_OUTOFMEMORY;
+
+    enumassoc->IEnumAssocHandlers_iface.lpVtbl = &enumassochandlersvtbl;
+    enumassoc->ref = 1;
+
+    *enumhandlers = &enumassoc->IEnumAssocHandlers_iface;
+    return S_OK;
+}
