@@ -428,8 +428,8 @@ static struct cpu_register cpu_register_map[] = {
     REG(X26, 8, CONTEXT_INTEGER),
     REG(X27, 8, CONTEXT_INTEGER),
     REG(X28, 8, CONTEXT_INTEGER),
-    REG(X29, 8, CONTEXT_INTEGER),
-    REG(X30, 8, CONTEXT_INTEGER),
+    REG(Fp,  8, CONTEXT_INTEGER),
+    REG(Lr,  8, CONTEXT_INTEGER),
     REG(Sp,  8, CONTEXT_CONTROL),
     REG(Pc,  8, CONTEXT_CONTROL),
     REG(PState, 8, CONTEXT_CONTROL),
