@@ -177,8 +177,10 @@ static const struct fd_ops dir_fd_ops =
 {
     dir_get_poll_events,         /* get_poll_events */
     default_poll_event,          /* poll_event */
-    no_flush,                    /* flush */
     dir_get_fd_type,             /* get_fd_type */
+    no_fd_read,                  /* read */
+    no_fd_write,                 /* write */
+    no_fd_flush,                 /* flush */
     default_fd_ioctl,            /* ioctl */
     default_fd_queue_async,      /* queue_async */
     default_fd_reselect_async,   /* reselect_async */
@@ -270,7 +272,7 @@ void sigio_callback(void)
     LIST_FOR_EACH_ENTRY( dir, &change_list, struct dir, entry )
     {
         if (interlocked_xchg( &dir->notified, 0 ))
-            fd_async_wake_up( dir->fd, ASYNC_TYPE_WAIT, STATUS_NOTIFY_ENUM_DIR );
+            fd_async_wake_up( dir->fd, ASYNC_TYPE_WAIT, STATUS_ALERTED );
     }
 }
 
